Package: crumblr
Type: Package
Title: Count ratio uncertainty modeling base linear regression
Version: 1.5.0
Date: 2025-03-11
Authors@R: person("Gabriel", "Hoffman", role=c("aut", "cre"), email="gabriel.hoffman@mssm.edu", comment=c(ORCID="0000-0002-0957-0224"))
Description: Crumblr enables analysis of count ratio data using precision weighted linear (mixed) models.  It uses an asymptotic normal approximation of the variance following the centered log ration transform (CLR) that is widely used in compositional data analysis.  Crumblr provides a fast, flexible alternative to GLMs and GLMM's while retaining high power and controlling the false positive rate.  
VignetteBuilder: knitr
License: Artistic-2.0
Encoding: UTF-8
URL: https://DiseaseNeurogenomics.github.io/crumblr
BugReports: https://github.com/DiseaseNeurogenomics/crumblr/issues
Suggests: BiocStyle, RUnit, knitr, rmarkdown, dreamlet, muscat,
        ExperimentHub, scater, HMP, reshape2, glue, tidyverse,
        BiocGenerics, compositions
biocViews: RNASeq, GeneExpression, DifferentialExpression, BatchEffect,
        QualityControl, SingleCell, Regression, Epigenetics,
        FunctionalGenomics, Transcriptomics, Normalization, Clustering,
        DimensionReduction, Preprocessing, Software
Depends: R (>= 4.4.0), ggplot2, methods
Imports: Rdpack, viridis, tidytree, variancePartition (>= 1.36.3),
        SingleCellExperiment, ggtree, dplyr, stats, MASS, Rfast
RoxygenNote: 7.3.2
RdMacros: Rdpack
LazyData: false
NeedsCompilation: no
git_url: https://git.bioconductor.org/packages/crumblr
git_branch: devel
git_last_commit: e19e122
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-11
Packaged: 2026-05-11 21:07:11 UTC; biocbuild
Author: Gabriel Hoffman [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0957-0224>)
Maintainer: Gabriel Hoffman <gabriel.hoffman@mssm.edu>
