# Generated by roxygen2: do not edit by hand

export(buildClusterTree)
export(clr)
export(clrInv)
export(crumblr)
export(diffTree)
export(dmn_mle)
export(logFrac)
export(meanSdPlot)
export(plotForest)
export(plotScatterDensity)
export(plotTreeTest)
export(plotTreeTestBeta)
export(standardize)
export(treeTest)
exportMethods(plotForest)
exportMethods(standardize)
import(ggplot2)
importFrom(MASS,kde2d)
importFrom(Rdpack,reprompt)
importFrom(Rfast,Digamma)
importFrom(Rfast,Lgamma)
importFrom(Rfast,colmeans)
importFrom(Rfast,rowsums)
importFrom(SingleCellExperiment,colData)
importFrom(SingleCellExperiment,reducedDim)
importFrom(dplyr,`%>%`)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,inner_join)
importFrom(dplyr,pull)
importFrom(dplyr,tibble)
importFrom(ggplot2,element_text)
importFrom(ggplot2,scale_color_gradient2)
importFrom(ggplot2,scale_size_area)
importFrom(ggplot2,theme)
importFrom(ggtree,geom_point2)
importFrom(ggtree,geom_text2)
importFrom(ggtree,geom_tiplab)
importFrom(ggtree,get_taxa_name)
importFrom(ggtree,ggtree)
importFrom(methods,new)
importFrom(stats,as.dist)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,p.adjust)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(tidytree,as.phylo)
importFrom(tidytree,as.treedata)
importFrom(tidytree,as_tibble)
importFrom(tidytree,left_join)
importFrom(tidytree,offspring)
importFrom(variancePartition,mvTest)
importFrom(viridis,scale_color_viridis)
